using System.Security.Claims;
using FitConnect.Models.v1.Api;
using Microsoft.IdentityModel.JsonWebTokens;
using Newtonsoft.Json;

namespace FitConnect.Models;

public enum EventType {
    NotSet,
    Create,
    Submit,
    Notify,
    Forward,
    Reject,
    Accept,
    Delete
}

public class SecurityEventToken {
    public SecurityEventToken(string jwtEncodedString) {
        Token = new JsonWebToken(jwtEncodedString);
        EventType = DecodeEventType(Token.Claims);
        var iat = Token.Claims.FirstOrDefault(c => c.Type == "iat").Value;
        if (long.TryParse(iat, out var timeEpoch))
            EventTime = DateTime.UnixEpoch.AddSeconds(timeEpoch);

        // TODO Fill problems
    }

    public DateTime EventTime { get; set; }
    public EventType EventType { get; set; }
    public List<Problems>? Problems { get; set; }
    public Events? Event { get; set; }

    public object? Payload { get; set; }
    public JsonWebToken Token { get; set; }

    private EventType DecodeEventType(IEnumerable<Claim> claims) {
        var eventsClaim = claims.FirstOrDefault(c => c.Type == "events");
        if (eventsClaim == null) return EventType.NotSet;

        var events = JsonConvert.DeserializeObject<Dictionary<string, dynamic>>(eventsClaim.Value);
        Payload = JsonConvert
            .DeserializeObject<dynamic>(
                events.Values.FirstOrDefault().ToString());


        if (eventsClaim.Value.Contains(
                "https://schema.fitko.de/fit-connect/events/create-submission"))
            return EventType.Create;

        if (eventsClaim.Value.Contains(
                "https://schema.fitko.de/fit-connect/events/submit-submission"))
            return EventType.Submit;

        if (eventsClaim.Value.Contains(
                "https://schema.fitko.de/fit-connect/events/notify-submission"))
            return EventType.Notify;
        if (eventsClaim.Value.Contains(
                "https://schema.fitko.de/fit-connect/events/forward-submission"))
            return EventType.Forward;
        if (eventsClaim.Value.Contains(
                "https://schema.fitko.de/fit-connect/events/reject-submission"))
            return EventType.Reject;
        if (eventsClaim.Value.Contains(
                "https://schema.fitko.de/fit-connect/events/accept-submission"))
            return EventType.Accept;
        if (eventsClaim.Value.Contains(
                "https://schema.fitko.de/fit-connect/events/delete-submission"))
            return EventType.Delete;

        return EventType.NotSet;
    }
}
